<?php
/*------------------------------------------------------------------------
# plg_starsnippets - StarSnippets Plugin
# ------------------------------------------------------------------------
# author    1sr
# copyright Copyright (C) 2013 1sr.de All Rights Reserved.
# @license - http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
# Websites: http://1sr.de
-------------------------------------------------------------------------*/

defined('JPATH_BASE') or die;

jimport('joomla.plugin.plugin');
jimport('joomla.factory');
jimport('joomla.error.profiler');

class plgSystemStarSnippets extends JPlugin {


	function __construct( &$subject, $params ) {
		
		parent::__construct( $subject, $params );

		$app = JFactory::getApplication();

		$this->admin = $this->params->get('admin', 0);
		
		// only run replacement on site, not admin area
		$this->shouldrun = !$app->isAdmin();

	}
	
	// replace using onAfterRender event
	function onAfterRender() {

		if ($this->shouldrun) {

			$body = JResponse::getBody();
			
			// use schema of current page 
			$domain = JURI::getInstance(JURI::current())->getScheme()."://star-snippets.com";
			
			$timeout = ($this->params->get('requestTimeout') != "" && is_int($this->params->get('requestTimeout'))) ? is_int($this->params->get('requestTimeout')) : 10;
			
			
			/*
			 * define configuration by joomla plugin
			 */
			
			$snippet = ($this->params->get('snippet') != "") ? $this->params->get('snippet') : "{star-snippet}";
			$ratingUrl = JURI::current();			
			$wrapAggregateRating = true;			
			$isAmp = false;
			
			$size = $this->params->get('size');
			$orientation = $this->params->get('orientation');
			$emptyColor = ($this->params->get('emptyColor') != "") ? " emptyColor=\"".$this->params->get('emptyColor')."\"" : "";
			$fillColor = ($this->params->get('fillColor') != "") ? " fillColor=\"".$this->params->get('fillColor')."\"" : "";
			$hoverColor = ($this->params->get('hoverColor') != "") ? " hoverColor=\"".$this->params->get('hoverColor')."\"" : "";
			$showRating = $this->params->get('showRating');
			
			/* 
			 * override configuration by snippet
			 *
			 * <span data-type="star-snippet" data-href="http://www.example.com" data-size="s" data-orientation="vertical" data-empty-color="333333" data-fill-color="96A9D5" data-hover-color="1A346F" data-show-rating="false" data-wrap-aggregate-rating="false" data-is-amp="true"></span>
			 */
			
			$snippetMatches = null;
			if(preg_match('/<span data-type="star-snippet".*?<\\/span>/', $body, $snippetMatches)) {
		
				$snippet = $snippetMatches[0];
		
				$hrefMatches = null;
				if(preg_match('/data-href=".*?"/', $snippet, $hrefMatches)) {
					$href = str_replace('"', '', str_replace('data-href="', '', $hrefMatches[0]));
					
					$ratingUrl = $href;
				}
		
				$sizeMatches = null;
				if(preg_match('/data-size=".*?"/', $snippet, $sizeMatches)) {
					$size = str_replace('"', '', str_replace('data-size="', '', $sizeMatches[0]));
				}

				$orientationMatches = null;
				if(preg_match('/data-orientation=".*?"/', $snippet, $orientationMatches)) {
					$orientation = str_replace('"', '', str_replace('data-orientation="', '', $orientationMatches[0]));
				}

				$emptyColorMatches = null;
				if(preg_match('/data-empty-color=".*?"/', $snippet, $emptyColorMatches)) {
					$emptyColor = str_replace('"', '', str_replace('data-empty-color="', '', $emptyColorMatches[0]));
					
					$emptyColor = " emptyColor=\"".$emptyColor."\"";
				}

				$fillColorMatches = null;
				if(preg_match('/data-fill-color=".*?"/', $snippet, $fillColorMatches)) {
					$fillColor = str_replace('"', '', str_replace('data-fill-color="', '', $fillColorMatches[0]));
					
					$fillColor = " fillColor=\"".$fillColor."\"";
				}

				$hoverColorMatches = null;
				if(preg_match('/data-hover-color=".*?"/', $snippet, $hoverColorMatches)) {
					$hoverColor = str_replace('"', '', str_replace('data-hover-color="', '', $hoverColorMatches[0]));
					
					$hoverColor = " hoverColor=\"".$hoverColor."\"";
				}

				$showRatingMatches = null;
				if(preg_match('/data-show-rating=".*?"/', $snippet, $showRatingMatches)) {
					$showRating = str_replace('"', '', str_replace('data-show-rating="', '', $showRatingMatches[0]));
				}
				
				$wrapAggregateRatingMatches = null;
				if(preg_match('/data-wrap-aggregate-rating=".*?"/', $snippet, $wrapAggregateRatingMatches)) {
					$wrapAggregateRating = str_replace('"', '', str_replace('data-wrap-aggregate-rating="', '', $wrapAggregateRatingMatches[0]));
					
					$wrapAggregateRating = ($wrapAggregateRating == "false") ? false : true;
				}
				
				$isAmpMatches = null;
				if(preg_match('/data-is-amp=".*?"/', $snippet, $isAmpMatches)) {
					$isAmp = str_replace('"', '', str_replace('data-is-amp="', '', $isAmpMatches[0]));
					
					$isAmp = ($isAmp == "true") ? true : false;
				}
		
			}
			
			$doc = JFactory::getDocument();
			 
			$url = $domain."/rating?url=".$ratingUrl;
			
			if($doc->getTitle() != "")
			{
			  $url .= "&name=".rawurlencode($doc->getTitle());
			}
			
			if($doc->getDescription() != "")
			{
			  $url .= "&description=".rawurlencode($doc->getDescription());
			}
			
			if(! $wrapAggregateRating)
			{
			  $url .= "&wrap=0";
			}
			
			$cache = JFactory::getCache('plg_starsnippets', '');
			$cache->setCaching(true);
			
			$cacheLifetime = ($this->params->get('cacheLifetime') != "") ? intval($this->params->get('cacheLifetime')) : 3600;
			
			$cache->setLifeTime($cacheLifetime);
			
			$cacheid = md5($this->params->get('requestFunction').'_'.$ratingUrl);
			
			
			if($this->params->get('requestFunction') == "file_get_contents")
			{
				try {
				
					$profiler = new JProfiler();
				
					// Get cached response
					$response = $cache->get($cacheid);
					
					if (empty($response)) {
					
						$streamContext = stream_context_create(array('http' => array('timeout' => $timeout)));				
						$response = file_get_contents($url, false, $streamContext);
						
					    $cache->store($response, $cacheid);
					    
					    //echo $profiler->mark( ' without caching' );
					}
					else {
					
						//echo $profiler->mark( ' without caching' );
						
					}

					if (!$response)
					{}
					else
					{
						if($isAmp){
							
							// echo "<pre class=\"hide\">".print_r($cache->getAll(), true)."</pre>";
				
							// remove style
							$response = preg_replace('/<div class="starSnippetsItemPropContainer" style="(.)*" /', '<div class="starSnippetsItemPropContainer" ', $response); 
							
							// remove starSnippetsAnchorContainer
							$response = preg_replace('/<div class="starSnippetsAnchorContainer"(.)*\\n(.)*\\n(.)*\\n(.)*\\n(.)*<\\/div>/', '', $response);
							
							// old star snippets
							
							$matches = null;
							if(preg_match('/<span itemprop="average">.*?<\\/span>/', $response, $matches)) {
								
								$average = preg_replace('/<span itemprop="average">/', '', $matches[0]);
								$average = preg_replace('/<\\/span>/', '', $average);
								
								$averagePercent = round(100*(floatval($average)/5));
								
								$body = str_replace($snippet, '<div class="star-rating-m"><div class="filled width-'.$averagePercent.'"><div class="cover"></div></div><div class="empty"><div class="cover"></div></div></div>'.$response, $body);
								
							}
							
							// new star snippets
							
							$matches = null;
							if(preg_match('/<meta itemprop="ratingValue" content=".*?" \\/>/', $response, $matches)) {
								
								$average = preg_replace('/<meta itemprop="ratingValue" content="/', '', $matches[0]);
								$average = preg_replace('/" \\/>/', '', $average);
								
								$averagePercent = round(100*(floatval($average)/5));
								
								$body = str_replace($snippet, '<div class="star-rating-m"><div class="filled width-'.$averagePercent.'"><div class="cover"></div></div><div class="empty"><div class="cover"></div></div></div>'.$response, $body);
								
							}
						}							
						else {
											
							$starSnippetTag = '<star-snippet'
								.' size="'.$size.'"'
								.' orientation="'.$orientation.'"'
								.$emptyColor
								.$fillColor
								.$hoverColor
								.' showRating="'.$showRating.'"'
								.'>'
								.'</star-snippet>';
							
							$body = str_replace($snippet, $starSnippetTag.$response, $body);
							
						}
					}
				} catch (Exception $ex) {
				}
			}
			else
			{
				try {
					
					$profiler = new JProfiler();
				
					// Get cached response
					$response = $cache->get($cacheid);
					
					if (empty($response)) {
					
						$session = curl_init($url);
						curl_setopt($session, CURLOPT_HEADER, false); //no header
						curl_setopt($session, CURLOPT_RETURNTRANSFER, true);
						curl_setopt($session, CURLOPT_TIMEOUT, $timeout);
						$response = curl_exec($session);
						curl_close($session);
						
					    $cache->store($response, $cacheid);
					    
					    //echo $profiler->mark( ' without caching' );
					}
					else {
					
						//echo $profiler->mark( ' without caching' );
						
					}

					if (!$response)
					{}
					else
					{
						if($isAmp){
						
							// remove style
							$response = preg_replace('/<div class="starSnippetsItemPropContainer" style="(.)*" /', '<div class="starSnippetsItemPropContainer" ', $response); 
							
							// remove starSnippetsAnchorContainer
							$response = preg_replace('/<div class="starSnippetsAnchorContainer"(.)*\\n(.)*\\n(.)*\\n(.)*\\n(.)*<\\/div>/', '', $response);
							
							// old star snippets
							
							$matches = null;
							if(preg_match('/<span itemprop="average">.*?<\\/span>/', $response, $matches)) {
								
								$average = preg_replace('/<span itemprop="average">/', '', $matches[0]);
								$average = preg_replace('/<\\/span>/', '', $average);
								
								$averagePercent = round(100*(floatval($average)/5));
								
								$body = str_replace($snippet, '<div class="star-rating-m"><div class="filled width-'.$averagePercent.'"><div class="cover"></div></div><div class="empty"><div class="cover"></div></div></div>'.$response, $body);
								
							}
							
							// new star snippets
							
							$matches = null;
							if(preg_match('/<meta itemprop="ratingValue" content=".*?" \\/>/', $response, $matches)) {
								
								$average = preg_replace('/<meta itemprop="ratingValue" content="/', '', $matches[0]);
								$average = preg_replace('/" \\/>/', '', $average);
								
								$averagePercent = round(100*(floatval($average)/5));
								
								$body = str_replace($snippet, '<div class="star-rating-m"><div class="filled width-'.$averagePercent.'"><div class="cover"></div></div><div class="empty"><div class="cover"></div></div></div>'.$response, $body);
								
							}
							
						}
						else {
						
							$starSnippetTag = '<star-snippet'
								.' size="'.$size.'"'
								.' orientation="'.$orientation.'"'
								.$emptyColor
								.$fillColor
								.$hoverColor
								.' showRating="'.$showRating.'"'
								.'>'
								.'</star-snippet>';
							
							$body = str_replace($snippet, $starSnippetTag.$response, $body);
							
						}
						
					}
				} catch (Exception $ex) {
				}
			}
			
			$jsCode = "\n"
				."<script>\n"
					."(function() {\n"
						."function async_load(){\n"						
							."var s = document.createElement('script');\n"
							."s.type = 'text/javascript';\n"
							."s.async = true;\n"
							."s.src = '".$domain."/script';\n"
							."//s.src = '".$domain."/script?prevent_cache=' + new Date().getTime(); // prevent caching\n"
							."var x = document.getElementsByTagName('script')[0];\n"
							."x.parentNode.insertBefore(s, x);\n"
						."}\n"
						."if (window.attachEvent) {\n"
							."window.attachEvent('onload', async_load);\n"
						."} else {"
							."window.addEventListener('load', async_load, false);}\n"
					."})();\n"
				."</script>\n";
				
			if(!$isAmp){
				
				$body = str_replace("</body>", $jsCode."</body>", $body);
			
			}

			JResponse::setBody($body);
		}			
	}
}